Module: Datadog::CI::Ext::Test
- Defined in:
- lib/datadog/ci/ext/test.rb
Overview
Defines constants for test tags
Defined Under Namespace
Constant Summary collapse
- CONTEXT_ORIGIN =
"ciapp-test"- TAG_FRAMEWORK =
Test visibility tags
"test.framework"- TAG_FRAMEWORK_VERSION =
"test.framework_version"- TAG_NAME =
"test.name"- TAG_SKIP_REASON =
"test.skip_reason"- TAG_STATUS =
"test.status"- TAG_SUITE =
"test.suite"- TAG_MODULE =
"test.module"- TAG_TYPE =
"test.type"- TAG_COMMAND =
"test.command"- TAG_SOURCE_FILE =
"test.source.file"- TAG_SOURCE_START =
"test.source.start"- TAG_CODEOWNERS =
"test.codeowners"- TAG_PARAMETERS =
"test.parameters"- TAG_CODE_COVERAGE_ENABLED =
Test optimisation tags
"test.code_coverage.enabled"- TAG_ITR_TEST_SKIPPING_ENABLED =
"test.itr.tests_skipping.enabled"- TAG_ITR_TEST_SKIPPING_TYPE =
"test.itr.tests_skipping.type"- TAG_ITR_TEST_SKIPPING_COUNT =
"test.itr.tests_skipping.count"- TAG_ITR_SKIPPED_BY_ITR =
"test.skipped_by_itr"- TAG_ITR_TESTS_SKIPPED =
"_dd.ci.itr.tests_skipped"- TAG_ITR_UNSKIPPABLE =
"test.itr.unskippable"- TAG_ITR_FORCED_RUN =
"test.itr.forced_run"- TAG_TEST_SESSION_ID =
Internal tags, they are not sent to the backend. These tags are internal to this library and used to correlate tests with the test sessions, suites, and modules.
"_test.session_id"- TAG_TEST_MODULE_ID =
"_test.module_id"- TAG_TEST_SUITE_ID =
"_test.suite_id"- TRANSIENT_TAGS =
[TAG_TEST_SESSION_ID, TAG_TEST_MODULE_ID, TAG_TEST_SUITE_ID].freeze
- TAG_OS_ARCHITECTURE =
Environment runtime tags
"os.architecture"- TAG_OS_PLATFORM =
"os.platform"- TAG_OS_VERSION =
"os.version"- TAG_RUNTIME_NAME =
"runtime.name"- TAG_RUNTIME_VERSION =
"runtime.version"- TAG_IS_RUM_ACTIVE =
Tags for browser tests
"test.is_rum_active"- TAG_BROWSER_DRIVER =
true if Datadog RUM was detected in the page(s) loaded by Selenium
"test.browser.driver"- TAG_BROWSER_DRIVER_VERSION =
version of selenium driver used
"test.browser.driver_version"- TAG_BROWSER_NAME =
name of the browser (Chrome, Firefox, Edge, etc), if multiple browsers then this tag is empty
"test.browser.name"- TAG_BROWSER_VERSION =
version of the browser, if multiple browsers or multiple versions then this tag is empty
"test.browser.version"- TAG_IS_RETRY =
Tags for retries
"test.is_retry"- TAG_IS_NEW =
true if test was retried by datadog-ci library
"test.is_new"- TAG_EARLY_FLAKE_ENABLED =
true if test was marked as new by new test retries (early flake detection)
"test.early_flake.enabled"- TAG_EARLY_FLAKE_ABORT_REASON =
true if early flake detection is enabled
"test.early_flake.abort_reason"- TAG_SPAN_KIND =
internal APM tag to mark a span as a test span
"span.kind"- SPAN_KIND_TEST =
"test"- INHERITABLE_TAGS =
tags that are common for the whole session and can be inherited from the test session
[TAG_FRAMEWORK, TAG_FRAMEWORK_VERSION].freeze
- ITR_TEST_SKIPPING_MODE =
could be either “test” or “suite” depending on whether we skip individual tests or whole suites
"test"- ITR_TEST_SKIP_REASON =
we always skip tests (not suites) in Ruby
"Skipped by Datadog's intelligent test runner"- ITR_UNSKIPPABLE_OPTION =
:datadog_itr_unskippable- EARLY_FLAKE_FAULTY =
"faulty"