Module: Datadog::CI::Ext::Test::LibraryConfigurationError
- Defined in:
- lib/datadog/ci/ext/test.rb
Overview
Hidden tags used to indicate that a communication error occurred with the backend during one of the library configuration requests. They are propagated to every CI event (test session, test module, test suite, test) so that the backend can detect when data may be incomplete because the library could not load configuration.
Constant Summary collapse
- TAG_SETTINGS =
"_dd.ci.library_configuration_error.settings"- TAG_SKIPPABLE_TESTS =
"_dd.ci.library_configuration_error.skippable_tests"- TAG_KNOWN_TESTS =
"_dd.ci.library_configuration_error.known_tests"- TAG_TEST_MANAGEMENT_TESTS =
"_dd.ci.library_configuration_error.test_management_tests"