Module: Datadog::CI::Ext::Test
- Defined in:
- lib/datadog/ci/ext/test.rb
Overview
Defines constants for test tags
Defined Under Namespace
Constant Summary collapse
- CONTEXT_ORIGIN =
"ciapp-test"- TAG_ARGUMENTS =
"test.arguments"- TAG_FRAMEWORK =
"test.framework"- TAG_FRAMEWORK_VERSION =
"test.framework_version"- TAG_NAME =
"test.name"- TAG_SKIP_REASON =
"test.skip_reason"- TAG_STATUS =
"test.status"- TAG_SUITE =
"test.suite"- TAG_MODULE =
"test.module"- TAG_TYPE =
"test.type"- TAG_COMMAND =
"test.command"- TAG_SOURCE_FILE =
"test.source.file"- TAG_SOURCE_START =
"test.source.start"- TAG_CODEOWNERS =
"test.codeowners"- TAG_PARAMETERS =
"test.parameters"- TAG_ITR_TEST_SKIPPING_ENABLED =
ITR tags
"test.itr.tests_skipping.enabled"- TAG_ITR_TEST_SKIPPING_TYPE =
"test.itr.tests_skipping.type"- TAG_CODE_COVERAGE_ENABLED =
Code coverage tags
"test.code_coverage.enabled"- TAG_TEST_SESSION_ID =
those tags are special and used to correlate tests with the test sessions, suites, and modules they are transient and not sent to the backend
"_test.session_id"- TAG_TEST_MODULE_ID =
"_test.module_id"- TAG_TEST_SUITE_ID =
"_test.suite_id"- TRANSIENT_TAGS =
[TAG_TEST_SESSION_ID, TAG_TEST_MODULE_ID, TAG_TEST_SUITE_ID].freeze
- INHERITABLE_TAGS =
tags that are common for the whole session and can be inherited from the test session
[TAG_FRAMEWORK, TAG_FRAMEWORK_VERSION].freeze
- TAG_OS_ARCHITECTURE =
Environment runtime tags
"os.architecture"- TAG_OS_PLATFORM =
"os.platform"- TAG_RUNTIME_NAME =
"runtime.name"- TAG_RUNTIME_VERSION =
"runtime.version"- TAG_SPAN_KIND =
internal APM tag to mark a span as a test span
"span.kind"- SPAN_KIND_TEST =
"test"- ITR_TEST_SKIPPING_MODE =
could be either “test” or “suite” depending on whether we skip individual tests or whole suites we use test skipping for Ruby
"test"