Module: Datadog::CI::Ext::Test

Defined in:
lib/datadog/ci/ext/test.rb

Overview

Defines constants for test tags

Defined Under Namespace

Modules: Status

Constant Summary collapse

CONTEXT_ORIGIN =
"ciapp-test"
TAG_ARGUMENTS =
"test.arguments"
TAG_FRAMEWORK =
"test.framework"
TAG_FRAMEWORK_VERSION =
"test.framework_version"
TAG_NAME =
"test.name"
TAG_SKIP_REASON =

DEV: Not populated yet

"test.skip_reason"
TAG_STATUS =
"test.status"
TAG_SUITE =
"test.suite"
TAG_MODULE =
"test.module"
TAG_TRAITS =
"test.traits"
TAG_TYPE =
"test.type"
TAG_COMMAND =
"test.command"
TAG_TEST_SESSION_ID =

those tags are special and they are used to correlate tests with the test sessions, suites, and modules

"_test.session_id"
TAG_TEST_MODULE_ID =
"_test.module_id"
TAG_TEST_SUITE_ID =
"_test.suite_id"
SPECIAL_TAGS =
[TAG_TEST_SESSION_ID, TAG_TEST_MODULE_ID, TAG_TEST_SUITE_ID].freeze
INHERITABLE_TAGS =

tags that can be inherited from the test session

[TAG_FRAMEWORK, TAG_FRAMEWORK_VERSION, TAG_TYPE].freeze
TAG_OS_ARCHITECTURE =

Environment runtime tags

"os.architecture"
TAG_OS_PLATFORM =
"os.platform"
TAG_RUNTIME_NAME =
"runtime.name"
TAG_RUNTIME_VERSION =
"runtime.version"
TAG_SPAN_KIND =
"span.kind"