Class: Datadog::CI::Span

Inherits:
Object
  • Object
show all
Defined in:
lib/datadog/ci/span.rb

Overview

Represents a single part of a test run. Could be a session, suite, test, or any custom span.

Direct Known Subclasses

ConcurrentSpan, Test

Instance Attribute Summary collapse

Instance Method Summary collapse

Constructor Details

#initialize(tracer_span) ⇒ Span

Returns a new instance of Span.



15
16
17
# File 'lib/datadog/ci/span.rb', line 15

def initialize(tracer_span)
  @tracer_span = tracer_span
end

Instance Attribute Details

#tracer_spanObject (readonly)

Returns the value of attribute tracer_span.



13
14
15
# File 'lib/datadog/ci/span.rb', line 13

def tracer_span
  @tracer_span
end

Instance Method Details

#failed!(exception: nil) ⇒ void

This method returns an undefined value.

Sets the status of the span to “fail”.

Parameters:

  • exception (Exception) (defaults to: nil)

    the exception that caused the test to fail.



72
73
74
75
76
# File 'lib/datadog/ci/span.rb', line 72

def failed!(exception: nil)
  tracer_span.status = 1
  tracer_span.set_tag(Ext::Test::TAG_STATUS, Ext::Test::Status::FAIL)
  tracer_span.set_error(exception) unless exception.nil?
end

#failed?bool

Checks whether span status is FAIL.

Returns:

  • (bool)

    true if span status is FAIL



53
54
55
# File 'lib/datadog/ci/span.rb', line 53

def failed?
  tracer_span.get_tag(Ext::Test::TAG_STATUS) == Ext::Test::Status::FAIL
end

#finishvoid

This method returns an undefined value.

Finishes the span.



113
114
115
# File 'lib/datadog/ci/span.rb', line 113

def finish
  tracer_span.finish
end

#get_tag(key) ⇒ String

Gets tag value by key.

Parameters:

  • key (String)

    the key of the tag.

Returns:

  • (String)

    the value of the tag.



91
92
93
# File 'lib/datadog/ci/span.rb', line 91

def get_tag(key)
  tracer_span.get_tag(key)
end

#git_branchString

Returns the git branch name extracted from the environment.

Returns:

  • (String)

    the branch.



138
139
140
# File 'lib/datadog/ci/span.rb', line 138

def git_branch
  tracer_span.get_tag(Ext::Git::TAG_BRANCH)
end

#git_commit_shaString

Returns the latest commit SHA extracted from the environment.

Returns:

  • (String)

    the commit SHA of the last commit.



132
133
134
# File 'lib/datadog/ci/span.rb', line 132

def git_commit_sha
  tracer_span.get_tag(Ext::Git::TAG_COMMIT_SHA)
end

#git_repository_urlString

Returns the git repository URL extracted from the environment.

Returns:

  • (String)

    the repository URL.



126
127
128
# File 'lib/datadog/ci/span.rb', line 126

def git_repository_url
  tracer_span.get_tag(Ext::Git::TAG_REPOSITORY_URL)
end

#idInteger

Returns the ID of the span.

Returns:

  • (Integer)

    the ID of the span.



20
21
22
# File 'lib/datadog/ci/span.rb', line 20

def id
  tracer_span.id
end

#nameString

Returns the name of the span.

Returns:

  • (String)

    the name of the span.



25
26
27
# File 'lib/datadog/ci/span.rb', line 25

def name
  tracer_span.name
end

#os_architectureString

Returns the OS architecture extracted from the environment.

Returns:

  • (String)

    OS arch.



144
145
146
# File 'lib/datadog/ci/span.rb', line 144

def os_architecture
  tracer_span.get_tag(Ext::Test::TAG_OS_ARCHITECTURE)
end

#os_platformString

Returns the OS platform extracted from the environment.

Returns:

  • (String)

    OS platform.



150
151
152
# File 'lib/datadog/ci/span.rb', line 150

def os_platform
  tracer_span.get_tag(Ext::Test::TAG_OS_PLATFORM)
end

#passed!void

This method returns an undefined value.

Sets the status of the span to “pass”.



65
66
67
# File 'lib/datadog/ci/span.rb', line 65

def passed!
  tracer_span.set_tag(Ext::Test::TAG_STATUS, Ext::Test::Status::PASS)
end

#passed?bool

Checks whether span status is PASS.

Returns:

  • (bool)

    true if span status is PASS



47
48
49
# File 'lib/datadog/ci/span.rb', line 47

def passed?
  tracer_span.get_tag(Ext::Test::TAG_STATUS) == Ext::Test::Status::PASS
end

#runtime_nameString

Returns the runtime name extracted from the environment.

Returns:

  • (String)

    runtime name.



156
157
158
# File 'lib/datadog/ci/span.rb', line 156

def runtime_name
  tracer_span.get_tag(Ext::Test::TAG_RUNTIME_NAME)
end

#runtime_versionString

Returns the runtime version extracted from the environment.

Returns:

  • (String)

    runtime version.



162
163
164
# File 'lib/datadog/ci/span.rb', line 162

def runtime_version
  tracer_span.get_tag(Ext::Test::TAG_RUNTIME_VERSION)
end

#serviceString

Returns the service name of the span.

Returns:

  • (String)

    the service name of the span.



30
31
32
# File 'lib/datadog/ci/span.rb', line 30

def service
  tracer_span.service
end

#set_default_tagsObject



174
175
176
# File 'lib/datadog/ci/span.rb', line 174

def set_default_tags
  tracer_span.set_tag(Ext::Test::TAG_SPAN_KIND, Ext::Test::SPAN_KIND_TEST)
end

#set_environment_runtime_tagsObject



166
167
168
169
170
171
172
# File 'lib/datadog/ci/span.rb', line 166

def set_environment_runtime_tags
  tracer_span.set_tag(Ext::Test::TAG_OS_ARCHITECTURE, ::RbConfig::CONFIG["host_cpu"])
  tracer_span.set_tag(Ext::Test::TAG_OS_PLATFORM, ::RbConfig::CONFIG["host_os"])
  tracer_span.set_tag(Ext::Test::TAG_RUNTIME_NAME, Core::Environment::Ext::LANG_ENGINE)
  tracer_span.set_tag(Ext::Test::TAG_RUNTIME_VERSION, Core::Environment::Ext::ENGINE_VERSION)
  tracer_span.set_tag(Ext::Test::TAG_COMMAND, Utils::TestRun.command)
end

#set_metric(key, value) ⇒ void

This method returns an undefined value.

Sets metric value by key.

Parameters:

  • key (String)

    the key of the metric.

  • value (Numeric)

    the value of the metric.



107
108
109
# File 'lib/datadog/ci/span.rb', line 107

def set_metric(key, value)
  tracer_span.set_metric(key, value)
end

#set_tag(key, value) ⇒ void

This method returns an undefined value.

Sets tag value by key.

Parameters:

  • key (String)

    the key of the tag.

  • value (String)

    the value of the tag.



99
100
101
# File 'lib/datadog/ci/span.rb', line 99

def set_tag(key, value)
  tracer_span.set_tag(key, value)
end

#set_tags(tags) ⇒ void

This method returns an undefined value.

Sets multiple tags at once.

Parameters:

  • tags (Hash[String, String])

    the tags to set.



120
121
122
# File 'lib/datadog/ci/span.rb', line 120

def set_tags(tags)
  tracer_span.set_tags(tags)
end

#skipped!(exception: nil, reason: nil) ⇒ void

This method returns an undefined value.

Sets the status of the span to “skip”.

Parameters:

  • exception (Exception) (defaults to: nil)

    the exception that caused the test to fail.

  • reason (String) (defaults to: nil)

    the reason why the test was skipped.



82
83
84
85
86
# File 'lib/datadog/ci/span.rb', line 82

def skipped!(exception: nil, reason: nil)
  tracer_span.set_tag(Ext::Test::TAG_STATUS, Ext::Test::Status::SKIP)
  tracer_span.set_error(exception) unless exception.nil?
  tracer_span.set_tag(Ext::Test::TAG_SKIP_REASON, reason) unless reason.nil?
end

#skipped?bool

Checks whether span status is SKIP.

Returns:

  • (bool)

    true if span status is SKIP



59
60
61
# File 'lib/datadog/ci/span.rb', line 59

def skipped?
  tracer_span.get_tag(Ext::Test::TAG_STATUS) == Ext::Test::Status::SKIP
end

#to_sObject



178
179
180
# File 'lib/datadog/ci/span.rb', line 178

def to_s
  "#{self.class}(name:#{name},tracer_span:#{@tracer_span})"
end

#typeString

Returns the type of the span (for example “test” or type that was provided to [Datadog::CI.trace]).

Returns:

  • (String)

    the type of the span (for example “test” or type that was provided to [Datadog::CI.trace]).



35
36
37
# File 'lib/datadog/ci/span.rb', line 35

def type
  tracer_span.type
end

#undefined?bool

Checks whether span status is not set yet.

Returns:

  • (bool)

    true if span does not have status



41
42
43
# File 'lib/datadog/ci/span.rb', line 41

def undefined?
  tracer_span.get_tag(Ext::Test::TAG_STATUS).nil?
end