Class: Datadog::CI::Span
- Inherits:
-
Object
- Object
- Datadog::CI::Span
- Defined in:
- lib/datadog/ci/span.rb
Overview
Represents a single part of a test run. Could be a session, suite, test, or any custom span.
Direct Known Subclasses
Instance Attribute Summary collapse
-
#tracer_span ⇒ Object
readonly
Returns the value of attribute tracer_span.
Instance Method Summary collapse
-
#failed!(exception: nil) ⇒ void
Sets the status of the span to “fail”.
-
#failed? ⇒ bool
Checks whether span status is FAIL.
-
#finish ⇒ void
Finishes the span.
-
#get_tag(key) ⇒ String
Gets tag value by key.
-
#git_branch ⇒ String
Returns the git branch name extracted from the environment.
-
#git_commit_sha ⇒ String
Returns the latest commit SHA extracted from the environment.
-
#git_repository_url ⇒ String
Returns the git repository URL extracted from the environment.
-
#id ⇒ Integer
The ID of the span.
-
#initialize(tracer_span) ⇒ Span
constructor
A new instance of Span.
-
#name ⇒ String
The name of the span.
-
#os_architecture ⇒ String
Returns the OS architecture extracted from the environment.
-
#os_platform ⇒ String
Returns the OS platform extracted from the environment.
-
#passed! ⇒ void
Sets the status of the span to “pass”.
-
#passed? ⇒ bool
Checks whether span status is PASS.
-
#runtime_name ⇒ String
Returns the runtime name extracted from the environment.
-
#runtime_version ⇒ String
Returns the runtime version extracted from the environment.
-
#service ⇒ String
The service name of the span.
- #set_default_tags ⇒ Object
- #set_environment_runtime_tags ⇒ Object
-
#set_metric(key, value) ⇒ void
Sets metric value by key.
-
#set_tag(key, value) ⇒ void
Sets tag value by key.
-
#set_tags(tags) ⇒ void
Sets multiple tags at once.
-
#skipped!(exception: nil, reason: nil) ⇒ void
Sets the status of the span to “skip”.
-
#skipped? ⇒ bool
Checks whether span status is SKIP.
- #to_s ⇒ Object
-
#type ⇒ String
The type of the span (for example “test” or type that was provided to [Datadog::CI.trace]).
-
#undefined? ⇒ bool
Checks whether span status is not set yet.
Constructor Details
#initialize(tracer_span) ⇒ Span
Returns a new instance of Span.
15 16 17 |
# File 'lib/datadog/ci/span.rb', line 15 def initialize(tracer_span) @tracer_span = tracer_span end |
Instance Attribute Details
#tracer_span ⇒ Object (readonly)
Returns the value of attribute tracer_span.
13 14 15 |
# File 'lib/datadog/ci/span.rb', line 13 def tracer_span @tracer_span end |
Instance Method Details
#failed!(exception: nil) ⇒ void
This method returns an undefined value.
Sets the status of the span to “fail”.
72 73 74 75 76 |
# File 'lib/datadog/ci/span.rb', line 72 def failed!(exception: nil) tracer_span.status = 1 tracer_span.set_tag(Ext::Test::TAG_STATUS, Ext::Test::Status::FAIL) tracer_span.set_error(exception) unless exception.nil? end |
#failed? ⇒ bool
Checks whether span status is FAIL.
53 54 55 |
# File 'lib/datadog/ci/span.rb', line 53 def failed? tracer_span.get_tag(Ext::Test::TAG_STATUS) == Ext::Test::Status::FAIL end |
#finish ⇒ void
This method returns an undefined value.
Finishes the span.
113 114 115 |
# File 'lib/datadog/ci/span.rb', line 113 def finish tracer_span.finish end |
#get_tag(key) ⇒ String
Gets tag value by key.
91 92 93 |
# File 'lib/datadog/ci/span.rb', line 91 def get_tag(key) tracer_span.get_tag(key) end |
#git_branch ⇒ String
Returns the git branch name extracted from the environment.
138 139 140 |
# File 'lib/datadog/ci/span.rb', line 138 def git_branch tracer_span.get_tag(Ext::Git::TAG_BRANCH) end |
#git_commit_sha ⇒ String
Returns the latest commit SHA extracted from the environment.
132 133 134 |
# File 'lib/datadog/ci/span.rb', line 132 def git_commit_sha tracer_span.get_tag(Ext::Git::TAG_COMMIT_SHA) end |
#git_repository_url ⇒ String
Returns the git repository URL extracted from the environment.
126 127 128 |
# File 'lib/datadog/ci/span.rb', line 126 def git_repository_url tracer_span.get_tag(Ext::Git::TAG_REPOSITORY_URL) end |
#id ⇒ Integer
Returns the ID of the span.
20 21 22 |
# File 'lib/datadog/ci/span.rb', line 20 def id tracer_span.id end |
#name ⇒ String
Returns the name of the span.
25 26 27 |
# File 'lib/datadog/ci/span.rb', line 25 def name tracer_span.name end |
#os_architecture ⇒ String
Returns the OS architecture extracted from the environment.
144 145 146 |
# File 'lib/datadog/ci/span.rb', line 144 def os_architecture tracer_span.get_tag(Ext::Test::TAG_OS_ARCHITECTURE) end |
#os_platform ⇒ String
Returns the OS platform extracted from the environment.
150 151 152 |
# File 'lib/datadog/ci/span.rb', line 150 def os_platform tracer_span.get_tag(Ext::Test::TAG_OS_PLATFORM) end |
#passed! ⇒ void
This method returns an undefined value.
Sets the status of the span to “pass”.
65 66 67 |
# File 'lib/datadog/ci/span.rb', line 65 def passed! tracer_span.set_tag(Ext::Test::TAG_STATUS, Ext::Test::Status::PASS) end |
#passed? ⇒ bool
Checks whether span status is PASS.
47 48 49 |
# File 'lib/datadog/ci/span.rb', line 47 def passed? tracer_span.get_tag(Ext::Test::TAG_STATUS) == Ext::Test::Status::PASS end |
#runtime_name ⇒ String
Returns the runtime name extracted from the environment.
156 157 158 |
# File 'lib/datadog/ci/span.rb', line 156 def runtime_name tracer_span.get_tag(Ext::Test::TAG_RUNTIME_NAME) end |
#runtime_version ⇒ String
Returns the runtime version extracted from the environment.
162 163 164 |
# File 'lib/datadog/ci/span.rb', line 162 def runtime_version tracer_span.get_tag(Ext::Test::TAG_RUNTIME_VERSION) end |
#service ⇒ String
Returns the service name of the span.
30 31 32 |
# File 'lib/datadog/ci/span.rb', line 30 def service tracer_span.service end |
#set_default_tags ⇒ Object
174 175 176 |
# File 'lib/datadog/ci/span.rb', line 174 def tracer_span.set_tag(Ext::Test::TAG_SPAN_KIND, Ext::Test::SPAN_KIND_TEST) end |
#set_environment_runtime_tags ⇒ Object
166 167 168 169 170 171 172 |
# File 'lib/datadog/ci/span.rb', line 166 def tracer_span.set_tag(Ext::Test::TAG_OS_ARCHITECTURE, ::RbConfig::CONFIG["host_cpu"]) tracer_span.set_tag(Ext::Test::TAG_OS_PLATFORM, ::RbConfig::CONFIG["host_os"]) tracer_span.set_tag(Ext::Test::TAG_RUNTIME_NAME, Core::Environment::Ext::LANG_ENGINE) tracer_span.set_tag(Ext::Test::TAG_RUNTIME_VERSION, Core::Environment::Ext::ENGINE_VERSION) tracer_span.set_tag(Ext::Test::TAG_COMMAND, Utils::TestRun.command) end |
#set_metric(key, value) ⇒ void
This method returns an undefined value.
Sets metric value by key.
107 108 109 |
# File 'lib/datadog/ci/span.rb', line 107 def set_metric(key, value) tracer_span.set_metric(key, value) end |
#set_tag(key, value) ⇒ void
This method returns an undefined value.
Sets tag value by key.
99 100 101 |
# File 'lib/datadog/ci/span.rb', line 99 def set_tag(key, value) tracer_span.set_tag(key, value) end |
#set_tags(tags) ⇒ void
This method returns an undefined value.
Sets multiple tags at once.
120 121 122 |
# File 'lib/datadog/ci/span.rb', line 120 def () tracer_span.() end |
#skipped!(exception: nil, reason: nil) ⇒ void
This method returns an undefined value.
Sets the status of the span to “skip”.
82 83 84 85 86 |
# File 'lib/datadog/ci/span.rb', line 82 def skipped!(exception: nil, reason: nil) tracer_span.set_tag(Ext::Test::TAG_STATUS, Ext::Test::Status::SKIP) tracer_span.set_error(exception) unless exception.nil? tracer_span.set_tag(Ext::Test::TAG_SKIP_REASON, reason) unless reason.nil? end |
#skipped? ⇒ bool
Checks whether span status is SKIP.
59 60 61 |
# File 'lib/datadog/ci/span.rb', line 59 def skipped? tracer_span.get_tag(Ext::Test::TAG_STATUS) == Ext::Test::Status::SKIP end |
#to_s ⇒ Object
178 179 180 |
# File 'lib/datadog/ci/span.rb', line 178 def to_s "#{self.class}(name:#{name},tracer_span:#{@tracer_span})" end |
#type ⇒ String
Returns the type of the span (for example “test” or type that was provided to [Datadog::CI.trace]).
35 36 37 |
# File 'lib/datadog/ci/span.rb', line 35 def type tracer_span.type end |
#undefined? ⇒ bool
Checks whether span status is not set yet.
41 42 43 |
# File 'lib/datadog/ci/span.rb', line 41 def undefined? tracer_span.get_tag(Ext::Test::TAG_STATUS).nil? end |