Class: Datadog::CI::TestOptimisation::Component

Inherits:
Object
  • Object
show all
Defined in:
lib/datadog/ci/test_optimisation/component.rb

Overview

Test Impact Analysis implementation Integrates with backend to provide test impact analysis data and skip tests that are not impacted by the changes

Instance Attribute Summary collapse

Instance Method Summary collapse

Constructor Details

#initialize(dd_env:, config_tags: {}, api: nil, coverage_writer: nil, enabled: false, bundle_location: nil, use_single_threaded_coverage: false, use_allocation_tracing: true) ⇒ Component

Returns a new instance of Component.



29
30
31
32
33
34
35
36
37
38
39
40
41
42
43
44
45
46
47
48
49
50
51
52
53
54
55
56
57
58
59
60
61
62
63
# File 'lib/datadog/ci/test_optimisation/component.rb', line 29

def initialize(
  dd_env:,
  config_tags: {},
  api: nil,
  coverage_writer: nil,
  enabled: false,
  bundle_location: nil,
  use_single_threaded_coverage: false,
  use_allocation_tracing: true
)
  @enabled = enabled
  @api = api
  @dd_env = dd_env
  @config_tags = config_tags || {}

  @bundle_location = if bundle_location && !File.absolute_path?(bundle_location)
    File.join(Git::LocalRepository.root, bundle_location)
  else
    bundle_location
  end
  @use_single_threaded_coverage = use_single_threaded_coverage
  @use_allocation_tracing = use_allocation_tracing

  @test_skipping_enabled = false
  @code_coverage_enabled = false

  @coverage_writer = coverage_writer

  @correlation_id = nil
  @skippable_tests = Set.new

  @mutex = Mutex.new

  Datadog.logger.debug("TestOptimisation initialized with enabled: #{@enabled}")
end

Instance Attribute Details

#code_coverage_enabledObject (readonly)

Returns the value of attribute code_coverage_enabled.



26
27
28
# File 'lib/datadog/ci/test_optimisation/component.rb', line 26

def code_coverage_enabled
  @code_coverage_enabled
end

#correlation_idObject (readonly)

Returns the value of attribute correlation_id.



26
27
28
# File 'lib/datadog/ci/test_optimisation/component.rb', line 26

def correlation_id
  @correlation_id
end

#enabledObject (readonly)

Returns the value of attribute enabled.



26
27
28
# File 'lib/datadog/ci/test_optimisation/component.rb', line 26

def enabled
  @enabled
end

#skippable_testsObject (readonly)

Returns the value of attribute skippable_tests.



26
27
28
# File 'lib/datadog/ci/test_optimisation/component.rb', line 26

def skippable_tests
  @skippable_tests
end

#skippable_tests_fetch_errorObject (readonly)

Returns the value of attribute skippable_tests_fetch_error.



26
27
28
# File 'lib/datadog/ci/test_optimisation/component.rb', line 26

def skippable_tests_fetch_error
  @skippable_tests_fetch_error
end

#test_skipping_enabledObject (readonly)

Returns the value of attribute test_skipping_enabled.



26
27
28
# File 'lib/datadog/ci/test_optimisation/component.rb', line 26

def test_skipping_enabled
  @test_skipping_enabled
end

Instance Method Details

#code_coverage?Boolean

Returns:

  • (Boolean)


94
95
96
# File 'lib/datadog/ci/test_optimisation/component.rb', line 94

def code_coverage?
  @code_coverage_enabled
end

#configure(remote_configuration, test_session) ⇒ Object



65
66
67
68
69
70
71
72
73
74
75
76
77
78
79
80
81
82
83
84
# File 'lib/datadog/ci/test_optimisation/component.rb', line 65

def configure(remote_configuration, test_session)
  return unless enabled?

  Datadog.logger.debug("Configuring TestOptimisation with remote configuration: #{remote_configuration}")

  @enabled = remote_configuration.itr_enabled?
  @test_skipping_enabled = @enabled && remote_configuration.tests_skipping_enabled?
  @code_coverage_enabled = @enabled && remote_configuration.code_coverage_enabled?

  test_session.set_tag(Ext::Test::TAG_ITR_TEST_SKIPPING_ENABLED, @test_skipping_enabled)
  test_session.set_tag(Ext::Test::TAG_CODE_COVERAGE_ENABLED, @code_coverage_enabled)
  # we skip tests, not suites
  test_session.set_tag(Ext::Test::TAG_ITR_TEST_SKIPPING_TYPE, Ext::Test::ITR_TEST_SKIPPING_MODE)

  load_datadog_cov! if @code_coverage_enabled

  Datadog.logger.debug("Configured TestOptimisation with enabled: #{@enabled}, skipping_tests: #{@test_skipping_enabled}, code_coverage: #{@code_coverage_enabled}")

  fetch_skippable_tests(test_session)
end

#enabled?Boolean

Returns:

  • (Boolean)


86
87
88
# File 'lib/datadog/ci/test_optimisation/component.rb', line 86

def enabled?
  @enabled
end

#mark_if_skippable(test) ⇒ Object



147
148
149
150
151
152
153
154
155
156
157
# File 'lib/datadog/ci/test_optimisation/component.rb', line 147

def mark_if_skippable(test)
  return if !enabled? || !skipping_tests?

  if skippable?(test)
    test.set_tag(Ext::Test::TAG_ITR_SKIPPED_BY_ITR, "true")

    Datadog.logger.debug { "Marked test as skippable: #{test.datadog_test_id}" }
  else
    Datadog.logger.debug { "Test is not skippable: #{test.datadog_test_id}" }
  end
end

#on_test_finished(test, context) ⇒ Object



159
160
161
162
163
164
165
# File 'lib/datadog/ci/test_optimisation/component.rb', line 159

def on_test_finished(test, context)
  return if !test.skipped? || !test.skipped_by_test_impact_analysis?

  Telemetry.itr_skipped

  context.incr_tests_skipped_by_tia_count
end

#shutdown!Object



181
182
183
# File 'lib/datadog/ci/test_optimisation/component.rb', line 181

def shutdown!
  @coverage_writer&.stop
end

#skippable?(test) ⇒ Boolean

Returns:

  • (Boolean)


141
142
143
144
145
# File 'lib/datadog/ci/test_optimisation/component.rb', line 141

def skippable?(test)
  return false if !enabled? || !skipping_tests?

  @skippable_tests.include?(test.datadog_test_id)
end

#skippable_tests_countObject



177
178
179
# File 'lib/datadog/ci/test_optimisation/component.rb', line 177

def skippable_tests_count
  skippable_tests.count
end

#skipping_tests?Boolean

Returns:

  • (Boolean)


90
91
92
# File 'lib/datadog/ci/test_optimisation/component.rb', line 90

def skipping_tests?
  @test_skipping_enabled
end

#start_coverage(test) ⇒ Object



98
99
100
101
102
103
# File 'lib/datadog/ci/test_optimisation/component.rb', line 98

def start_coverage(test)
  return if !enabled? || !code_coverage?

  Telemetry.code_coverage_started(test)
  coverage_collector&.start
end

#stop_coverage(test) ⇒ Object



105
106
107
108
109
110
111
112
113
114
115
116
117
118
119
120
121
122
123
124
125
126
127
128
129
130
131
132
133
134
135
136
137
138
139
# File 'lib/datadog/ci/test_optimisation/component.rb', line 105

def stop_coverage(test)
  return if !enabled? || !code_coverage?

  Telemetry.code_coverage_finished(test)

  coverage = coverage_collector&.stop

  # if test was skipped, we discard coverage data
  return if test.skipped?

  if coverage.nil? || coverage.empty?
    Telemetry.code_coverage_is_empty
    return
  end

  test_source_file = test.source_file

  # cucumber's gherkin files are not covered by the code coverage collector
  ensure_test_source_covered(test_source_file, coverage) unless test_source_file.nil?

  Telemetry.code_coverage_files(coverage.size)

  event = Coverage::Event.new(
    test_id: test.id.to_s,
    test_suite_id: test.test_suite_id.to_s,
    test_session_id: test.test_session_id.to_s,
    coverage: coverage
  )

  Datadog.logger.debug { "Writing coverage event \n #{event.pretty_inspect}" }

  write(event)

  event
end

#write_test_session_tags(test_session, skipped_tests_count) ⇒ Object



167
168
169
170
171
172
173
174
175
# File 'lib/datadog/ci/test_optimisation/component.rb', line 167

def write_test_session_tags(test_session, skipped_tests_count)
  return if !enabled?

  Datadog.logger.debug { "Finished optimised session with test skipping enabled: #{@test_skipping_enabled}" }
  Datadog.logger.debug { "#{skipped_tests_count} tests were skipped" }

  test_session.set_tag(Ext::Test::TAG_ITR_TESTS_SKIPPED, skipped_tests_count.positive?.to_s)
  test_session.set_tag(Ext::Test::TAG_ITR_TEST_SKIPPING_COUNT, skipped_tests_count)
end