Class: Datadog::CI::TestVisibility::Recorder

Inherits:
Object
  • Object
show all
Defined in:
lib/datadog/ci/test_visibility/recorder.rb

Overview

Common behavior for CI tests Note: this class has too many responsibilities and should be split into multiple classes

Instance Attribute Summary collapse

Instance Method Summary collapse

Constructor Details

#initialize(itr:, remote_settings_api:, test_suite_level_visibility_enabled: false, codeowners: Codeowners::Parser.new(Utils::Git.root).parse) ⇒ Recorder

Returns a new instance of Recorder.



31
32
33
34
35
36
37
38
39
40
41
42
43
44
45
# File 'lib/datadog/ci/test_visibility/recorder.rb', line 31

def initialize(
  itr:, remote_settings_api:, test_suite_level_visibility_enabled: false,
  codeowners: Codeowners::Parser.new(Utils::Git.root).parse
)
  @test_suite_level_visibility_enabled = test_suite_level_visibility_enabled

  @environment_tags = Ext::Environment.tags(ENV).freeze
  @local_context = Context::Local.new
  @global_context = Context::Global.new

  @codeowners = codeowners

  @itr = itr
  @remote_settings_api = remote_settings_api
end

Instance Attribute Details

#environment_tagsObject (readonly)

Returns the value of attribute environment_tags.



29
30
31
# File 'lib/datadog/ci/test_visibility/recorder.rb', line 29

def environment_tags
  @environment_tags
end

#test_suite_level_visibility_enabledObject (readonly)

Returns the value of attribute test_suite_level_visibility_enabled.



29
30
31
# File 'lib/datadog/ci/test_visibility/recorder.rb', line 29

def test_suite_level_visibility_enabled
  @test_suite_level_visibility_enabled
end

Instance Method Details

#active_spanObject



149
150
151
152
# File 'lib/datadog/ci/test_visibility/recorder.rb', line 149

def active_span
  tracer_span = Datadog::Tracing.active_span
  Span.new(tracer_span) if tracer_span
end

#active_testObject



154
155
156
# File 'lib/datadog/ci/test_visibility/recorder.rb', line 154

def active_test
  @local_context.active_test
end

#active_test_moduleObject



162
163
164
# File 'lib/datadog/ci/test_visibility/recorder.rb', line 162

def active_test_module
  @global_context.active_test_module
end

#active_test_sessionObject



158
159
160
# File 'lib/datadog/ci/test_visibility/recorder.rb', line 158

def active_test_session
  @global_context.active_test_session
end

#active_test_suite(test_suite_name) ⇒ Object



166
167
168
# File 'lib/datadog/ci/test_visibility/recorder.rb', line 166

def active_test_suite(test_suite_name)
  @global_context.active_test_suite(test_suite_name)
end

#deactivate_testObject



170
171
172
# File 'lib/datadog/ci/test_visibility/recorder.rb', line 170

def deactivate_test
  @local_context.deactivate_test
end

#deactivate_test_moduleObject



178
179
180
# File 'lib/datadog/ci/test_visibility/recorder.rb', line 178

def deactivate_test_module
  @global_context.deactivate_test_module!
end

#deactivate_test_sessionObject



174
175
176
# File 'lib/datadog/ci/test_visibility/recorder.rb', line 174

def deactivate_test_session
  @global_context.deactivate_test_session!
end

#deactivate_test_suite(test_suite_name) ⇒ Object



182
183
184
# File 'lib/datadog/ci/test_visibility/recorder.rb', line 182

def deactivate_test_suite(test_suite_name)
  @global_context.deactivate_test_suite!(test_suite_name)
end

#itr_enabled?Boolean

Returns:

  • (Boolean)


186
187
188
# File 'lib/datadog/ci/test_visibility/recorder.rb', line 186

def itr_enabled?
  @itr.enabled?
end

#start_test_module(test_module_name, service: nil, tags: {}) ⇒ Object



64
65
66
67
68
69
70
71
72
73
74
75
76
77
78
# File 'lib/datadog/ci/test_visibility/recorder.rb', line 64

def start_test_module(test_module_name, service: nil, tags: {})
  return skip_tracing unless test_suite_level_visibility_enabled

  @global_context.fetch_or_activate_test_module do
    set_inherited_globals(tags)
    set_session_context(tags)

    tracer_span = start_datadog_tracer_span(
      test_module_name, build_span_options(service, Ext::AppTypes::TYPE_TEST_MODULE)
    )
    set_module_context(tags, tracer_span)

    build_test_module(tracer_span, tags)
  end
end

#start_test_session(service: nil, tags: {}) ⇒ Object



47
48
49
50
51
52
53
54
55
56
57
58
59
60
61
62
# File 'lib/datadog/ci/test_visibility/recorder.rb', line 47

def start_test_session(service: nil, tags: {})
  return skip_tracing unless test_suite_level_visibility_enabled

  @global_context.fetch_or_activate_test_session do
    tracer_span = start_datadog_tracer_span(
      "test.session", build_span_options(service, Ext::AppTypes::TYPE_TEST_SESSION)
    )
    set_session_context(tags, tracer_span)

    test_session = build_test_session(tracer_span, tags)

    configure_library(test_session)

    test_session
  end
end

#start_test_suite(test_suite_name, service: nil, tags: {}) ⇒ Object



80
81
82
83
84
85
86
87
88
89
90
91
92
93
94
95
# File 'lib/datadog/ci/test_visibility/recorder.rb', line 80

def start_test_suite(test_suite_name, service: nil, tags: {})
  return skip_tracing unless test_suite_level_visibility_enabled

  @global_context.fetch_or_activate_test_suite(test_suite_name) do
    set_inherited_globals(tags)
    set_session_context(tags)
    set_module_context(tags)

    tracer_span = start_datadog_tracer_span(
      test_suite_name, build_span_options(service, Ext::AppTypes::TYPE_TEST_SUITE)
    )
    set_suite_context(tags, span: tracer_span)

    build_test_suite(tracer_span, tags)
  end
end

#trace(span_name, type: "span", tags: {}, &block) ⇒ Object



131
132
133
134
135
136
137
138
139
140
141
142
143
144
145
146
147
# File 'lib/datadog/ci/test_visibility/recorder.rb', line 131

def trace(span_name, type: "span", tags: {}, &block)
  span_options = build_span_options(
    nil, # service name is completely optional for custom spans
    type,
    {resource: span_name}
  )

  if block
    start_datadog_tracer_span(span_name, span_options) do |tracer_span|
      block.call(build_span(tracer_span, tags))
    end
  else
    tracer_span = start_datadog_tracer_span(span_name, span_options)

    build_span(tracer_span, tags)
  end
end

#trace_test(test_name, test_suite_name, service: nil, tags: {}, &block) ⇒ Object



97
98
99
100
101
102
103
104
105
106
107
108
109
110
111
112
113
114
115
116
117
118
119
120
121
122
123
124
125
126
127
128
129
# File 'lib/datadog/ci/test_visibility/recorder.rb', line 97

def trace_test(test_name, test_suite_name, service: nil, tags: {}, &block)
  set_inherited_globals(tags)
  set_session_context(tags)
  set_module_context(tags)
  set_suite_context(tags, name: test_suite_name)

  tags[Ext::Test::TAG_NAME] = test_name
  tags[Ext::Test::TAG_TYPE] ||= Ext::Test::Type::TEST

  span_options = build_span_options(
    service,
    Ext::AppTypes::TYPE_TEST,
    # :resource is needed for the agent APM protocol to work correctly (for older agent versions)
    # :continue_from is required to start a new trace for each test
    {resource: test_name, continue_from: Datadog::Tracing::TraceDigest.new}
  )

  if block
    start_datadog_tracer_span(test_name, span_options) do |tracer_span|
      test = build_test(tracer_span, tags)

      @local_context.activate_test(test) do
        block.call(test)
      end
    end
  else
    tracer_span = start_datadog_tracer_span(test_name, span_options)

    test = build_test(tracer_span, tags)
    @local_context.activate_test(test)
    test
  end
end