Class: Datadog::CI::TestVisibility::Recorder

Inherits:
Object
  • Object
show all
Defined in:
lib/datadog/ci/test_visibility/recorder.rb

Overview

Common behavior for CI tests Note: this class has too many responsibilities and should be split into multiple classes

Instance Attribute Summary collapse

Instance Method Summary collapse

Constructor Details

#initialize(test_suite_level_visibility_enabled: false) ⇒ Recorder

Returns a new instance of Recorder.



30
31
32
33
34
35
36
# File 'lib/datadog/ci/test_visibility/recorder.rb', line 30

def initialize(test_suite_level_visibility_enabled: false)
  @test_suite_level_visibility_enabled = test_suite_level_visibility_enabled

  @environment_tags = Ext::Environment.tags(ENV).freeze
  @local_context = Context::Local.new
  @global_context = Context::Global.new
end

Instance Attribute Details

#environment_tagsObject (readonly)

Returns the value of attribute environment_tags.



28
29
30
# File 'lib/datadog/ci/test_visibility/recorder.rb', line 28

def environment_tags
  @environment_tags
end

#test_suite_level_visibility_enabledObject (readonly)

Returns the value of attribute test_suite_level_visibility_enabled.



28
29
30
# File 'lib/datadog/ci/test_visibility/recorder.rb', line 28

def test_suite_level_visibility_enabled
  @test_suite_level_visibility_enabled
end

Instance Method Details

#active_spanObject



135
136
137
138
# File 'lib/datadog/ci/test_visibility/recorder.rb', line 135

def active_span
  tracer_span = Datadog::Tracing.active_span
  Span.new(tracer_span) if tracer_span
end

#active_testObject



140
141
142
# File 'lib/datadog/ci/test_visibility/recorder.rb', line 140

def active_test
  @local_context.active_test
end

#active_test_moduleObject



148
149
150
# File 'lib/datadog/ci/test_visibility/recorder.rb', line 148

def active_test_module
  @global_context.active_test_module
end

#active_test_sessionObject



144
145
146
# File 'lib/datadog/ci/test_visibility/recorder.rb', line 144

def active_test_session
  @global_context.active_test_session
end

#active_test_suite(test_suite_name) ⇒ Object



152
153
154
# File 'lib/datadog/ci/test_visibility/recorder.rb', line 152

def active_test_suite(test_suite_name)
  @global_context.active_test_suite(test_suite_name)
end

#deactivate_test(test) ⇒ Object



156
157
158
# File 'lib/datadog/ci/test_visibility/recorder.rb', line 156

def deactivate_test(test)
  @local_context.deactivate_test!(test)
end

#deactivate_test_moduleObject



164
165
166
# File 'lib/datadog/ci/test_visibility/recorder.rb', line 164

def deactivate_test_module
  @global_context.deactivate_test_module!
end

#deactivate_test_sessionObject



160
161
162
# File 'lib/datadog/ci/test_visibility/recorder.rb', line 160

def deactivate_test_session
  @global_context.deactivate_test_session!
end

#deactivate_test_suite(test_suite_name) ⇒ Object



168
169
170
# File 'lib/datadog/ci/test_visibility/recorder.rb', line 168

def deactivate_test_suite(test_suite_name)
  @global_context.deactivate_test_suite!(test_suite_name)
end

#start_test_module(test_module_name, service: nil, tags: {}) ⇒ Object



51
52
53
54
55
56
57
58
59
60
61
62
63
64
65
# File 'lib/datadog/ci/test_visibility/recorder.rb', line 51

def start_test_module(test_module_name, service: nil, tags: {})
  return skip_tracing unless test_suite_level_visibility_enabled

  @global_context.fetch_or_activate_test_module do
    set_inherited_globals(tags)
    set_session_context(tags)

    tracer_span = start_datadog_tracer_span(
      test_module_name, build_span_options(service, Ext::AppTypes::TYPE_TEST_MODULE)
    )
    set_module_context(tags, tracer_span)

    build_test_module(tracer_span, tags)
  end
end

#start_test_session(service: nil, tags: {}) ⇒ Object



38
39
40
41
42
43
44
45
46
47
48
49
# File 'lib/datadog/ci/test_visibility/recorder.rb', line 38

def start_test_session(service: nil, tags: {})
  return skip_tracing unless test_suite_level_visibility_enabled

  @global_context.fetch_or_activate_test_session do
    tracer_span = start_datadog_tracer_span(
      "test.session", build_span_options(service, Ext::AppTypes::TYPE_TEST_SESSION)
    )
    set_session_context(tags, tracer_span)

    build_test_session(tracer_span, tags)
  end
end

#start_test_suite(test_suite_name, service: nil, tags: {}) ⇒ Object



67
68
69
70
71
72
73
74
75
76
77
78
79
80
81
82
# File 'lib/datadog/ci/test_visibility/recorder.rb', line 67

def start_test_suite(test_suite_name, service: nil, tags: {})
  return skip_tracing unless test_suite_level_visibility_enabled

  @global_context.fetch_or_activate_test_suite(test_suite_name) do
    set_inherited_globals(tags)
    set_session_context(tags)
    set_module_context(tags)

    tracer_span = start_datadog_tracer_span(
      test_suite_name, build_span_options(service, Ext::AppTypes::TYPE_TEST_SUITE)
    )
    set_suite_context(tags, span: tracer_span)

    build_test_suite(tracer_span, tags)
  end
end

#trace(span_type, span_name, tags: {}, &block) ⇒ Object



117
118
119
120
121
122
123
124
125
126
127
128
129
130
131
132
133
# File 'lib/datadog/ci/test_visibility/recorder.rb', line 117

def trace(span_type, span_name, tags: {}, &block)
  span_options = build_span_options(
    nil, # service name is completely optional for custom spans
    span_type,
    {resource: span_name}
  )

  if block
    start_datadog_tracer_span(span_name, span_options) do |tracer_span|
      block.call(build_span(tracer_span, tags))
    end
  else
    tracer_span = start_datadog_tracer_span(span_name, span_options)

    build_span(tracer_span, tags)
  end
end

#trace_test(test_name, test_suite_name, service: nil, tags: {}, &block) ⇒ Object



84
85
86
87
88
89
90
91
92
93
94
95
96
97
98
99
100
101
102
103
104
105
106
107
108
109
110
111
112
113
114
115
# File 'lib/datadog/ci/test_visibility/recorder.rb', line 84

def trace_test(test_name, test_suite_name, service: nil, tags: {}, &block)
  set_inherited_globals(tags)
  set_session_context(tags)
  set_module_context(tags)
  set_suite_context(tags, name: test_suite_name)

  tags[Ext::Test::TAG_NAME] = test_name

  span_options = build_span_options(
    service,
    Ext::AppTypes::TYPE_TEST,
    # :resource is needed for the agent APM protocol to work correctly (for older agent versions)
    # :continue_from is required to start a new trace for each test
    {resource: test_name, continue_from: Datadog::Tracing::TraceDigest.new}
  )

  if block
    start_datadog_tracer_span(test_name, span_options) do |tracer_span|
      test = build_test(tracer_span, tags)

      @local_context.activate_test!(test) do
        block.call(test)
      end
    end
  else
    tracer_span = start_datadog_tracer_span(test_name, span_options)

    test = build_test(tracer_span, tags)
    @local_context.activate_test!(test)
    test
  end
end